

#property link      "50pips"

#property indicator_separate_window

// Indicator's window fixed level properties

#property indicator_level1 0.0
#property indicator_levelcolor DimGray
#property indicator_levelstyle 0
#property indicator_levelwidth 1

#property indicator_buffers 8

// Indicator's line colors

#property indicator_color1 Black // MACD line color
#property indicator_color2 White // MAForStDev color
#property indicator_color3 MediumSeaGreen // MACDUp symbol color
#property indicator_color4 Red // MACDDown symbol color
#property indicator_color5 MediumSeaGreen // MACDAbove upper band symbol color
#property indicator_color6 Red // MACDBelow lower band symbol color
#property indicator_color7 Blue // UpperBand line color
#property indicator_color8 Red // LowerBand line color

// Indicator's symbol sizes different from 0

#property indicator_width3 1 // MACDUp symbol size
#property indicator_width4 1 // MACDDown symbol size
#property indicator_width5 1 // MACDAbove upper band symbol size
#property indicator_width6 1 // MACDBelow lower band symbol size

// Constants

#define SIGNAL_PERIOD 9

// Input parameters

extern int FastMAPeriod=12;
extern int SlowMAPeriod=26;
extern bool ShowMACDLine=false;
extern string MAMethodDecription="<Simple=0,Exponential=1>";
extern string MAMethodCont="<Smoothed=2,LinearWeighted=3>";
extern int MAMethodForStdDev=0;
extern int StdDevPeriod=10;
extern double StdDevCount=1.0;
extern bool ShowMAForStdDev=true;
extern string MACDSymbolCodes="<Dot=159,ThreeArmStar=169>";
extern string MACDSymbolCont="<Diamond=115,FullCircle=108>";
extern int MACDUpSymbol=108;
extern int MACDDownSymbol=108;
extern int MACDAboveSymbol=115;
extern int MACDBelowSymbol=115;
extern string ZeroArrowSettings="<Settings for Zero-Arrow>";
extern int ArrowSize=20;
extern color ZeroUpColor=MediumSeaGreen;
extern color ZeroDownColor=Red;
extern bool ShowZeroArrow=true;

extern bool DrawBox=true;
extern int XBox=200;
extern int YBox=-55;
//extern color BoxColor=White;

// Buffers

double MACDBuffer[];
double MAForStdDevBuffer[];
double MACDUpBuffer[];
double MACDDownBuffer[];
double MACDAboveBuffer[];
double MACDBelowBuffer[];
double UpperBandBuffer[];
double LowerBandBuffer[];

// Variables

string shortName="";

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init(){  
   ObjectDelete("line1");
   ObjectDelete("line2");
   ObjectDelete("line3");
   ObjectDelete("line4");
   ObjectDelete("line5");
   ObjectDelete("bands");
   ObjectDelete("bands0");
   ObjectDelete("bands1");
   ObjectDelete("bands2");
   ObjectDelete("arrow0");
   ObjectDelete("arrow1");
   ObjectDelete("arrow2");
   ObjectDelete("dots");
   ObjectDelete("bar");
   ObjectDelete("bar0");
   ObjectDelete("bar1");
   ObjectDelete("bar2");
   ObjectDelete("phigh");
   ObjectDelete("plow");
   
   // MACD buffer(line)settings.

   SetIndexBuffer(0,MACDBuffer);
   if(ShowMACDLine==true){
      SetIndexStyle(0,DRAW_LINE);   
   }else{
      SetIndexStyle(0,DRAW_NONE);
   }
   SetIndexLabel(0,"MACD("+SlowMAPeriod+","+FastMAPeriod+","+SIGNAL_PERIOD+")");

   // MAForStdDevBuffer buffer(line)settings.

   SetIndexBuffer(1,MAForStdDevBuffer);
   if(ShowMAForStdDev==true){
      SetIndexStyle(1,DRAW_LINE,STYLE_DASHDOTDOT);   
   }else{
      SetIndexStyle(1,DRAW_NONE);
   }
   SetIndexLabel(1,"MA For StdDev");
   
   // MACDUp buffer(symbols)settings.
  
   SetIndexBuffer(2,MACDUpBuffer);
   SetIndexStyle(2,DRAW_ARROW);
   SetIndexArrow(2,MACDUpSymbol);    
   SetIndexEmptyValue(2,EMPTY_VALUE);
   SetIndexLabel(2,"MACD Up");
   
   // MACDDown buffer(symbols)settings.

   SetIndexBuffer(3,MACDDownBuffer);
   SetIndexStyle(3,DRAW_ARROW);
   SetIndexArrow(3,MACDUpSymbol);  
   SetIndexEmptyValue(3,EMPTY_VALUE);
   SetIndexLabel(3,"MACD Down");
   
   // MACDAbove upper band buffer(symbols)settings.

   SetIndexBuffer(4,MACDAboveBuffer);
   SetIndexStyle(4,DRAW_ARROW);
   SetIndexArrow(4,MACDAboveSymbol);  
   SetIndexEmptyValue(4,EMPTY_VALUE);
   SetIndexLabel(4,"MACD Above");
   
   // MACDBelow upper band buffer(symbols)settings.

   SetIndexBuffer(5,MACDBelowBuffer);
   SetIndexStyle(5,DRAW_ARROW);
   SetIndexArrow(5,MACDBelowSymbol);  
   SetIndexEmptyValue(5,EMPTY_VALUE);
   SetIndexLabel(5,"MACD Below");
   
   // UpperBand buffer(line)settings.

   SetIndexBuffer(6,UpperBandBuffer);
   SetIndexStyle(6,DRAW_LINE);  
   SetIndexLabel(6,"Upper Band");
   
   // LowerBand buffer(line)settings.

   SetIndexBuffer(7,LowerBandBuffer);
   SetIndexStyle(7,DRAW_LINE);
   SetIndexLabel(7,"Lower Band");
   
   // General settings
   
   shortName="MACD_BB";
   IndicatorShortName(shortName);
   IndicatorDigits(Digits+1);
   
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+

//int deinit(){
 //  // Clear all created objects
 //  ObjectsDeleteAll();
//
//   return(0);
//}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int start(){
   int counted_bars=IndicatorCounted();
   if(counted_bars>0){
      counted_bars--;  
   }
   int limit=(Bars-1)-counted_bars;

   // Calculate MACD values and copy to related buffer(array).
 
   for(int barIndex=limit;barIndex>=0;barIndex--){
      MACDBuffer[barIndex]=iMACD(NULL,0,FastMAPeriod,SlowMAPeriod,SIGNAL_PERIOD,PRICE_CLOSE,MODE_MAIN,barIndex);      
   }  

   // Calculate MAForStdDevBuffer values and copy to related buffer(array).
   // Calculate upper and lower bands and copy to related buffer(array).    

   double stdDev=0.0;
   for(barIndex=limit;barIndex>=0;barIndex--){
      MAForStdDevBuffer[barIndex]=iMAOnArray(MACDBuffer,0,StdDevPeriod,0,MAMethodForStdDev,barIndex);
      stdDev=iStdDevOnArray(MACDBuffer,0,StdDevPeriod,0,MAMethodForStdDev,barIndex);
      UpperBandBuffer[barIndex]=MAForStdDevBuffer[barIndex]+(StdDevCount*stdDev);
      LowerBandBuffer[barIndex]=MAForStdDevBuffer[barIndex] -(StdDevCount*stdDev);
   }       

   // Determine MACD symbols and zero arrows.

   determineSymbols(limit);

   if(ShowZeroArrow==true){
      determineZeroArrow();
   }
   if(DrawBox){
      color PressureColor0=White;
      int textmargin0=0;
      color PressureColor1=White;
      int textmargin1=0;
      color PressureColor2=White;
      int textmargin2=0;
      double Multiplier = MathPow(10, Digits+1);
      double pressure0=Multiplier*(UpperBandBuffer[0]-LowerBandBuffer[0]);
      double pressure1=Multiplier*(UpperBandBuffer[1]-LowerBandBuffer[1]);
      double pressure2=Multiplier*(UpperBandBuffer[2]-LowerBandBuffer[2]);
      double pressure3=Multiplier*(UpperBandBuffer[3]-LowerBandBuffer[3]);
      if(pressure0>=pressure1){PressureColor0=MediumSeaGreen;textmargin0=3;}
      else{PressureColor0=Red;}
      if(pressure1>=pressure2){PressureColor1=MediumSeaGreen;textmargin1=3;}
      else{PressureColor1=Red;}
      if(pressure2>=pressure3){PressureColor2=MediumSeaGreen;textmargin2=3;}
      else{PressureColor2=Red;}
      
      ObjectCreate("bar",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("bar","Bar",10,"Tahoma Narrow",Gray);
      ObjectSet("bar",OBJPROP_CORNER,3);
      ObjectSet("bar",OBJPROP_XDISTANCE,XBox+100);
      ObjectSet("bar",OBJPROP_YDISTANCE,YBox+125);
      
      ObjectCreate("bands",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("bands","Bands",10,"Tahoma Narrow",Gray);
      ObjectSet("bands",OBJPROP_CORNER,3);
      ObjectSet("bands",OBJPROP_XDISTANCE,XBox+51);
      ObjectSet("bands",OBJPROP_YDISTANCE,YBox+125);
      
      ObjectCreate("dots",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("dots","Dots",10,"Tahoma Narrow",Gray);
      ObjectSet("dots",OBJPROP_CORNER,3);
      ObjectSet("dots",OBJPROP_XDISTANCE,XBox+15);
      ObjectSet("dots",OBJPROP_YDISTANCE,YBox+125);
      
      if(MACDBuffer[0]>=MAForStdDevBuffer[0]){
         ObjectCreate("arrow0", OBJ_LABEL,WindowFind(shortName),0,0);
         ObjectSetText("arrow0","",12,"Wingdings",MediumSeaGreen);
         ObjectSet("arrow0",OBJPROP_CORNER,3);
         ObjectSet("arrow0",OBJPROP_XDISTANCE,XBox+19);
         ObjectSet("arrow0",OBJPROP_YDISTANCE,YBox+103);
      }
      else{
         ObjectCreate("arrow0", OBJ_LABEL,WindowFind(shortName),0,0);
         ObjectSetText("arrow0","",12,"Wingdings",Red);
         ObjectSet("arrow0",OBJPROP_CORNER,3);
         ObjectSet("arrow0",OBJPROP_XDISTANCE,XBox+19);
         ObjectSet("arrow0",OBJPROP_YDISTANCE,YBox+100);
      }
      if(MACDBuffer[1]>=MAForStdDevBuffer[1]){
         ObjectCreate("arrow1", OBJ_LABEL,WindowFind(shortName),0,0);
         ObjectSetText("arrow1","",12,"Wingdings",MediumSeaGreen);
         ObjectSet("arrow1",OBJPROP_CORNER,3);
         ObjectSet("arrow1",OBJPROP_XDISTANCE,XBox+19);
         ObjectSet("arrow1",OBJPROP_YDISTANCE,YBox+87);
      }
      else{
         ObjectCreate("arrow1", OBJ_LABEL,WindowFind(shortName),0,0);
         ObjectSetText("arrow1","",12,"Wingdings",Red);
         ObjectSet("arrow1",OBJPROP_CORNER,3);
         ObjectSet("arrow1",OBJPROP_XDISTANCE,XBox+19);
         ObjectSet("arrow1",OBJPROP_YDISTANCE,YBox+84);
      }
      if(MACDBuffer[2]>=MAForStdDevBuffer[2]){
         ObjectCreate("arrow2", OBJ_LABEL,WindowFind(shortName),0,0);
         ObjectSetText("arrow2","",12,"Wingdings",MediumSeaGreen);
         ObjectSet("arrow2",OBJPROP_CORNER,3);
         ObjectSet("arrow2",OBJPROP_XDISTANCE,XBox+19);
         ObjectSet("arrow2",OBJPROP_YDISTANCE,YBox+71);
      }
      else{
         ObjectCreate("arrow2", OBJ_LABEL,WindowFind(shortName),0,0);
         ObjectSetText("arrow2","",12,"Wingdings",Red);
         ObjectSet("arrow2",OBJPROP_CORNER,3);
         ObjectSet("arrow2",OBJPROP_XDISTANCE,XBox+19);
         ObjectSet("arrow2",OBJPROP_YDISTANCE,YBox+68);
      }
      
      ObjectCreate("bar0",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("bar0","0",10,"Tahoma Narrow",Gray);
      ObjectSet("bar0",OBJPROP_CORNER,3);
      ObjectSet("bar0",OBJPROP_XDISTANCE,XBox+107);
      ObjectSet("bar0",OBJPROP_YDISTANCE,YBox+103);
      
      ObjectCreate("bar1",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("bar1","1",10,"Tahoma Narrow",Gray);
      ObjectSet("bar1",OBJPROP_CORNER,3);
      ObjectSet("bar1",OBJPROP_XDISTANCE,XBox+107);
      ObjectSet("bar1",OBJPROP_YDISTANCE,YBox+86);
      
      ObjectCreate("bar2",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("bar2","2",10,"Tahoma Narrow",Gray);
      ObjectSet("bar2",OBJPROP_CORNER,3);
      ObjectSet("bar2",OBJPROP_XDISTANCE,XBox+107);
      ObjectSet("bar2",OBJPROP_YDISTANCE,YBox+69);
      
      ObjectCreate("bands0",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("bands0",DoubleToStr(pressure0,0),10,"Tahoma Narrow",PressureColor0);
      ObjectSet("bands0",OBJPROP_CORNER,3);
      ObjectSet("bands0",OBJPROP_XDISTANCE,XBox+55+textmargin0);
      ObjectSet("bands0",OBJPROP_YDISTANCE,YBox+103);
      
      ObjectCreate("bands1",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("bands1",DoubleToStr(pressure1,0),10,"Tahoma Narrow",PressureColor1);
      ObjectSet("bands1",OBJPROP_CORNER,3);
      ObjectSet("bands1",OBJPROP_XDISTANCE,XBox+55+textmargin1);
      ObjectSet("bands1",OBJPROP_YDISTANCE,YBox+86);
      
      ObjectCreate("bands2",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("bands2",DoubleToStr(pressure2,0),10,"Tahoma Narrow",PressureColor2);
      ObjectSet("bands2",OBJPROP_CORNER,3);
      ObjectSet("bands2",OBJPROP_XDISTANCE,XBox+55+textmargin2);
      ObjectSet("bands2",OBJPROP_YDISTANCE,YBox+69);
      
      ObjectCreate("line1",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("line1","--------------------",9,"Arial",Gray);
      ObjectSet("line1",OBJPROP_CORNER,3);
      ObjectSet("line1",OBJPROP_ANGLE,90);
      ObjectSet("line1",OBJPROP_XDISTANCE,XBox+127);
      ObjectSet("line1",OBJPROP_YDISTANCE,YBox+146);
      
      ObjectCreate("line2",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("line2","--------------------",9,"Arial",Gray);
      ObjectSet("line2",OBJPROP_CORNER,3);
      ObjectSet("line2",OBJPROP_ANGLE,90);
      ObjectSet("line2",OBJPROP_XDISTANCE,XBox+2);
      ObjectSet("line2",OBJPROP_YDISTANCE,YBox+146);
      
      ObjectCreate("line3",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("line3","-------------------------------",9,"Arial",Gray);
      ObjectSet("line3",OBJPROP_CORNER,3);
      ObjectSet("line3",OBJPROP_XDISTANCE,XBox+8);
      ObjectSet("line3",OBJPROP_YDISTANCE,YBox+60);
      
      ObjectCreate("line4",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("line4","-------------------------------",9,"Arial",Gray);
      ObjectSet("line4",OBJPROP_CORNER,3);
      ObjectSet("line4",OBJPROP_XDISTANCE,XBox+8);
      ObjectSet("line4",OBJPROP_YDISTANCE,YBox+138);
      
      ObjectCreate("line5",OBJ_LABEL,WindowFind(shortName),0,0);
      ObjectSetText("line5","-------------------------------",9,"Arial",Gray);
      ObjectSet("line5",OBJPROP_CORNER,3);
      ObjectSet("line5",OBJPROP_XDISTANCE,XBox+8);
      ObjectSet("line5",OBJPROP_YDISTANCE,YBox+116);
   }
   return(0);
}

// Determine above and below band symbols and MACD up and down symbols and related buffers.
   
void determineSymbols(int limit){
   for(int barIndex=limit;barIndex>=0;barIndex--){       
      if(MACDBuffer[barIndex]>UpperBandBuffer[barIndex]){
         if(MACDBuffer[barIndex]>=MACDBuffer[barIndex+1]){
            MACDAboveBuffer[barIndex]=MACDBuffer[barIndex];
            MACDBelowBuffer[barIndex]=EMPTY_VALUE;           
         } else { 
            MACDBelowBuffer[barIndex]=MACDBuffer[barIndex];
            MACDAboveBuffer[barIndex]=EMPTY_VALUE;                 
         }    
         MACDUpBuffer[barIndex]=EMPTY_VALUE;
         MACDDownBuffer[barIndex]=EMPTY_VALUE;
      } else if(MACDBuffer[barIndex]<LowerBandBuffer[barIndex]){
         if(MACDBuffer[barIndex]<=MACDBuffer[barIndex+1]){
            MACDBelowBuffer[barIndex]=MACDBuffer[barIndex];
            MACDAboveBuffer[barIndex]=EMPTY_VALUE;           
         } else { 
            MACDAboveBuffer[barIndex]=MACDBuffer[barIndex];
            MACDBelowBuffer[barIndex]=EMPTY_VALUE;                 
         }    
         MACDUpBuffer[barIndex]=EMPTY_VALUE;
         MACDDownBuffer[barIndex]=EMPTY_VALUE;      
      } else if((MACDBuffer[barIndex]<=UpperBandBuffer[barIndex])&&
        (MACDBuffer[barIndex]>=LowerBandBuffer[barIndex])){
         // Determine MACDUp and MACDDown symbols. If previous and current MACD 
         // values are equal,use the previous bar to determine.  
         if(MACDBuffer[barIndex]>MACDBuffer[barIndex+1]){
            MACDUpBuffer[barIndex]=MACDBuffer[barIndex];
            MACDDownBuffer[barIndex]=EMPTY_VALUE;  
         } else if(MACDBuffer[barIndex]<MACDBuffer[barIndex+1]){
            MACDDownBuffer[barIndex]=MACDBuffer[barIndex];
            MACDUpBuffer[barIndex]=EMPTY_VALUE;
         } else if(MACDBuffer[barIndex]==MACDBuffer[barIndex+1]){
            if(MACDUpBuffer[barIndex+1]!=EMPTY_VALUE){
               MACDUpBuffer[barIndex]=MACDBuffer[barIndex];
               MACDDownBuffer[barIndex]=EMPTY_VALUE;
            } else if(MACDDownBuffer[barIndex+1]!=EMPTY_VALUE){
               MACDDownBuffer[barIndex]=MACDBuffer[barIndex];
               MACDUpBuffer[barIndex]=EMPTY_VALUE;        
            } else if((MACDUpBuffer[barIndex+1]==EMPTY_VALUE)&& 
              (MACDDownBuffer[barIndex+1]==EMPTY_VALUE)){
               if(MACDAboveBuffer[barIndex+1]!=EMPTY_VALUE){
                  MACDUpBuffer[barIndex]=MACDBuffer[barIndex];
                  MACDDownBuffer[barIndex]=EMPTY_VALUE;
               }else if(MACDBelowBuffer[barIndex+1]!=EMPTY_VALUE){
                  MACDDownBuffer[barIndex]=MACDBuffer[barIndex];
                  MACDUpBuffer[barIndex]=EMPTY_VALUE;
               }       
            }
         }
         MACDAboveBuffer[barIndex]=EMPTY_VALUE;
         MACDBelowBuffer[barIndex]=EMPTY_VALUE;                      
      }      
   }
}

// Determine arrows which show MACD crossing 0(zero)line whether to up or to down. 
// Nothing is displayed if MACD equals to 0(zero).

void determineZeroArrow(){   
   int windowIndex=WindowFind(shortName);
   if(MACDBuffer[0]>0.0){
      ObjectDelete("zeroDown");
      ObjectCreate("zeroUp",OBJ_LABEL,windowIndex,0,0);
      ObjectSet("zeroUp",OBJPROP_CORNER,3); 
      ObjectSet("zeroUp",OBJPROP_XDISTANCE,5);
      ObjectSet("zeroUp",OBJPROP_YDISTANCE,10);
      ObjectSetText("zeroUp",CharToStr(233),ArrowSize,"Wingdings",ZeroUpColor);
   }else if(MACDBuffer[0]<0.0){ 
      ObjectDelete("zeroUp");
      ObjectCreate("zeroDown",OBJ_LABEL,windowIndex,0,0);
      ObjectSet("zeroDown",OBJPROP_CORNER,1); 
      ObjectSet("zeroDown",OBJPROP_XDISTANCE,5);
      ObjectSet("zeroDown",OBJPROP_YDISTANCE,10);
      ObjectSetText("zeroDown",CharToStr(234),ArrowSize,"Wingdings",ZeroDownColor);
   }else{
      ObjectDelete("zeroUp");
      ObjectDelete("zeroDown");     
   }   
}